function [time, perf] = SGD(X, Y, tau, testX, testY, budget)
% Stochastic Gradient Descent for solving multinomial logistic regression
% under a trace norm constraint
    
%   Input: 
%   X = feature vectors    
%   Y = labels
%   tau = trace norm bound

%   Output:     
    
    N = 25000000;
    m = 100;
    [n, d] = size(X);
    h = max(Y);
    W = zeros(h,d);
    
    step = 50;
    time = [];
    perf = [];
    time_elapsed = 0;
    
    tic;
    for k = 1:N
        G = get_grad(W,X,Y,randsample(n,m)');
        %G = get_grad(W,X,Y,1:n);
        
        eta = 1 / sqrt(k);
        %W = W - eta * G;
        W = TraceProject(W - eta * G, tau);                
        
        if mod(k, step) == 0
            time_elapsed = time_elapsed + toc;
            time = [time; time_elapsed];
            perf = [perf; measure(W, X, Y, testX, testY)];
            fprintf('SGD: k = %d done\n', k);
            
            if time_elapsed > budget                 
                return
            end
            tic;
        end
    end
        
end
